
/*
* Send comments to:
*   Dave Kuhlman
*   Reify
*   426 Madison Ave #9
*   Redwood City, CA 94061
*   415-368-8450
*   Internet: dkuhlman@netcom.com
*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

// Get the correct form of function prototypes in flen3.he.
#if ! defined(__IBMCPP__)
#define __ANSI__
#endif // __IBMCPP__

#include <stdio.h>
#include <string.h>

// #include <isynonym.hpp>

#include "mdlsyn.hpp"
#include "mdl.hpp"
#include "sup.hpp"
#include "db.hpp"
#include "bt.h"

// Constants:

#define MIN_REC 10
#define MAX_REC 4096
#define GRANULARITY 16

// Global variables used only in this file:

static VFILE *Dbfd;
static BTFILE *DbfdKey;


// --------------------------------------------------------

Mdl::ErrStat btInitialize()
{

    DbfdKey = NULL;
    Dbfd = NULL;
    return(Mdl::ERR_STAT_OK);
} // btInitialize 

// --------------------------------------------------------

Mdl::ErrStat btTerminate()
{

    // If a DB is open, close it.
    if ( DbfdKey != NULL )
    {
        lioClose( DbfdKey );
        vlClose( Dbfd );
        DbfdKey = NULL;
        Dbfd = NULL;
    } // if

    return (Mdl::ERR_STAT_OK);
} // btTerminate 

// --------------------------------------------------------

#define KEYSIZE 64
#define KEYTYPE tSTRING

// Create a new database.

Boolean btCreateDb( char *name )
{
    char    filename[128];
    char    buf[257];
    sfint   reply;

    // Create the index file.
    sprintf( filename, "%s%s", name, ".KEY" );
    reply = lioCreat( filename, KEYSIZE, KEYTYPE, 0, 1 );
    if ( reply == (sfint)ERROR )
    {
        sprintf(buf, "btCreateDb: error.  name = %s\n", filename );
        MdlSupport::ReportError(buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return( False );
    } // if 

    // Create the data file.
    strcpy( filename, name );
    reply = vlCreat( filename, MIN_REC, MAX_REC, GRANULARITY, MAX_REC );
    if ( reply == (sfint)ERROR )
    {
        sprintf(buf, "btCreateDb: error.  name = %s\n", filename );
        MdlSupport::ReportError(buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return( False );
    } // if 

    if ( ! btOpenDb( name ) )
    {
        return( False );
    } // if 

    return( True );
} // btCreateDb 

// --------------------------------------------------------

// Open an existing database.
//  There is only one DB open at a time.
//  If another DB is open when this function is called, close it.

Boolean btOpenDb( char *name )
{
    char    filename[128];
    char    buf[256];

    // If a DB is open, close it.
    if ( DbfdKey != NULL )
    {
        lioClose( DbfdKey );
        vlClose( Dbfd );
        DbfdKey = NULL;
        Dbfd = NULL;
    } // if 

    // Open the index file:
    sprintf( filename, "%s%s", name, ".KEY" );
    DbfdKey = lioOpen(
            filename,
            READWRITE );
    if ( DbfdKey == (BTFILE *)NULL )
    {
        sprintf(buf, "btOpenDb: error.  name = %s\n", filename );
        MdlSupport::ReportError(buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return( False );
    } // if 

    // Open the data file:
    strcpy( filename, name );
    if ( ( Dbfd = vlOpen( filename, READWRITE ) ) == (VFILE *)ERROR )
    {
        sprintf(buf, "btOpenDb: error.  name = %s\n", filename );
        MdlSupport::ReportError( buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return( False );
    } // if 

    return( True );
} // btOpenDb 

// --------------------------------------------------------

// Close the database.

void btCloseDb()
{

    if ( DbfdKey == NULL )
    {
        return;
    } // if 

    lioClose( DbfdKey );
    vlClose( Dbfd );
    DbfdKey = NULL;
    Dbfd = NULL;
} // btCloseDb 

// --------------------------------------------------------

Db::RecNumber btAddRec( void *pBuf, short size )
{
    sfuint      reply;
    Db::RecNumber   recno;
    char        buf[256];

    recno = vlAvl( Dbfd );
    if ( recno == (Db::RecNumber)ERROR )
    {
        sprintf(buf, "btAddRec: error 1.  recno = %ld\n", recno);
        MdlSupport::ReportError( buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return(Db::DB_NULL_RECID);
    } // if 

    reply = vlWrite( Dbfd, recno, (char *)pBuf, size );
    if ( reply == (sfuint)ERROR )
    {
        sprintf(buf, "btAddRec: error 2.  recno = %ld\n", recno);
        MdlSupport::ReportError( buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return(Db::DB_NULL_RECID);
    } // if 

    return( recno );
} // btAddRec 

// --------------------------------------------------------

Db::RecNumber btWriteRec( Db::RecNumber recno, void *pBuf, short size )
{
    char        buf[256];
    sfuint      reply;

    reply = vlWrite( Dbfd, recno, (char *)pBuf, size );
    if ( reply == (sfuint)ERROR )
    {

        sprintf(buf, "btWriteRec: error.  recno = %ld\n", recno);
        MdlSupport::ReportError( buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return(Db::DB_NULL_RECID);
    } // if 

    return( recno );
} // btWriteRec

// --------------------------------------------------------

Boolean btReadRec(
        Db::RecNumber   recno,
        void        *pBuf
        )
{
    char        buf[256];
    sfuint      reply;

    reply = vlRead( Dbfd, recno, (char *)pBuf, 0 );
    if ( reply == (sfuint)ERROR )
    {
        sprintf(buf, "btReadRec: error.  recno = %ld\n", recno);
        MdlSupport::ReportError( buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return( False );
    } // if 

    return( True );
} // btReadRec 


// --------------------------------------------------------

Boolean btDeleteRec( Db::RecNumber recno )
{
    char        buf[256];
    sfuint      reply;

    reply = vlFree( Dbfd, recno );
    if ( reply == (sfuint)ERROR )
    {
        sprintf(buf, "btDeleteRec: error.  recno = %ld\n", recno);
        MdlSupport::ReportError( buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return( False );
    } // if 

    return( True );
} // btDeleteRec 

// --------------------------------------------------------


Db::RecNumber btFirstKey( char *key )
{
    Db::RecNumber   recno;
    char        buf[256];

    recno = lioFirst( DbfdKey, key );
    if ( recno == (Db::RecNumber)ERROR )
    {
        sprintf(buf, "btFirstKey: error.  recno = %ld\n", recno);
        MdlSupport::ReportError( buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return(Db::DB_NULL_RECID);
    } // if 

    return( recno );
} // btFirstKey 

// --------------------------------------------------------

Db::RecNumber btNextKey( char *key )
{
    Db::RecNumber   recno;
    char        buf[256];

    recno = lioNext( DbfdKey, key );
    if ( recno == (Db::RecNumber)ERROR )
    {
        sprintf(buf, "btNextKey: error.  recno = %ld\n", recno);
        MdlSupport::ReportError( buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return(Db::DB_NULL_RECID);
    } // if 

    return( recno );
} // btNextKey 

// --------------------------------------------------------

Db::RecNumber btSearchKey( char *key )
{
    Db::RecNumber   recno;
    char        buf[256];

    recno = lioSearch( DbfdKey, key );
    if ( recno == (Db::RecNumber)ERROR )
    {
        return(Db::DB_NULL_RECID);
    } // if 

    return( recno );
} // btSearchKey 

// --------------------------------------------------------

Boolean btAddKey( char *key, Db::RecNumber recno )
{
    char    buf[256];

    if ( lioInsert( DbfdKey, key, recno ) == (Db::RecNumber)ERROR )
    {
        sprintf(buf, "btAddKey: error.  recno = %ld\n", recno);
        MdlSupport::ReportError( buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return( False );
    } // if 

    return( True );
} // btAddKey 

// --------------------------------------------------------

Boolean btDeleteKey( char *key )
{
    char    buf[256];

    if ( lioDelete( DbfdKey, key ) == (Db::RecNumber)ERROR )
    {
        sprintf(buf, "btDeleteKey: error.  key = %s\n", key );
        MdlSupport::ReportError(buf, MdlSupport::MDL_ERROR_LEVEL_1);
        return( False );
    } // if 

    return( True );
} // btDeleteKey 

// --------------------------------------------------------


