// dbblk.hpp    Class definitions for DbBlock

/*
* Send comments to:
*   Dave Kuhlman
*   Reify
*   426 Madison Ave #9
*   Redwood City, CA 94061
*   415-368-8450
*   Internet: dkuhlman@netcom.com
*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#ifndef dbblk_HPP
#define dbblk_HPP

#include "mdlsyn.hpp"

#define BLKBLKSIZE          2048
#define BLKNOPINS           64
#define BLKNULLPIN          -1

struct BlkBlkHdRec {
    Db::RecNumber   nextBlk;
    Db::RecNumber   headRecno;  // If this is the first of a series of blocks,
                                //  then this field contains the RecId of
                                //  the head (first) record in this set
                                //  of blocks.  Else null.
    short       nextFreePin;
    short       pins[BLKNOPINS];
    short       nextFreeStore;
};
typedef BlkBlkHdRec * BlkBlkHdRecPtr;

#define BLKSIZESTORE ( BLKBLKSIZE - sizeof(BlkBlkHdRec) )

struct BlkBlkRec {
    BlkBlkRec *     next;
    BlkBlkHdRec     h;                   // Header
    char            store[BLKSIZESTORE]; // Store for strings
};
typedef BlkBlkRec * BlkBlkRecPtr;

typedef struct BLKRECNOREC {
    short   blkNo;
    short   pinNo;
} BlkRecnoRec, *BlkRecnoRecPtr;

typedef struct BLKSTOREREC {
    short   size;
    char    data[1];
} BlkStoreRec, *BlkStoreRecPtr;

class DbBlock {
public:
    DbBlock();
    ~DbBlock();

    // Add a string to the current block.
    //  If the current string block is full, write it out and initialize another.
    //  Generate an error if we exceed the maximum number of string blocks.
    Db::RecNumber AddDbString(const char * str);
    Db::RecNumber AddDbString(char * str);
    // Add a record to the current string block.
    //  If the current string block is full, write it out and initialize another.
    //  Generate an error if we exceed the maximum number of string blocks.
//    Db::RecNumber AddDbRec(void *pRec, short len);
    Db::RecNumber AddDbRec(
        void    *pRec,
        short   len,
        Boolean descriptor = False,
        char * className = NULL
    );
    // Return a pointer to a string within an array of string blocks that
    //  have been loaded from the database.
    char * GetStr(Db::RecNumber strId);
    // Find a record within an array of string blocks that
    //  have been loaded from the database.
    //  Copy the record into the requestor's buffer.
    void GetRec(
        Db::RecNumber   recId,      // IN: String block record ID
        void        *pBuf       // OUT: Destination for data
    );
    // Write  a record back into an array of DB blocks.
    void WriteDbRec(
        Db::RecNumber   recId,      // IN: Block record ID
        void        *pBuf,      // IN: Ptr to data
        short       size        // IN: Size of data.  Ignored.  Use old size
    );
    // Delete a set of (chained) blocks from the database.
    void DeleteDbRecs(
            Db::RecNumber   recId       /* IN: Record ID of first block */
            );
    // Flush the DB blocks in the array to disk and return the record number of
    //  the first block in the array.
    Db::RecNumber FlushBlocks(Db::RecNumber headRecno);
    // Load a list of blocks into an array of blocks.
    //  This array of blocks is the array in which strFindStr() searches.
    Db::RecNumber LoadBlocks(Db::RecNumber firstRecno);
    static Boolean DbBlock::IsDescriptor(Db::RecNumber recno, Db::RecNumber &convertedRecno);
protected:
private:
    BlkBlkRec * data;
    int blkIndex;

    // Private functions:
    void initBlk(void);
    short addRecToBlk(void *pRec, short len);
    short addStrToBlk(const char *pStr);
    short alignLength(short len);

    // The copy constructor is private so no one can use it.
    DbBlock(const DbBlock& rhs);
    // The assignment operator is private so no one can use it.
    DbBlock& operator =(const DbBlock& rhs);
}; // DbBlock

#endif // dbblk_HPP

