// dbdir.cpp    Member functions for DbDirDirectory, DbDirModel

/*
* Send comments to:
*   Dave Kuhlman
*   Reify
*   426 Madison Ave #9
*   Redwood City, CA 94061
*   415-368-8450
*   Internet: dkuhlman@netcom.com
*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#include <string.h>

#include "dbdir.hpp"

// --------------------------------------------------------

// Private constants:

// Private globals:

// Private function prototypes:

// --------------------------------------------------------

DbDirDirectory::DbDirDirectory()
:   name(NULL), firstChild(NULL), lastChild(NULL), sibling(NULL)
{
} // DbDirDirectory::DbDirDirectory

// --------------------------------------------------------

DbDirDirectory::DbDirDirectory(char * initName)
:   firstChild(NULL), lastChild(NULL), sibling(NULL)
{
    name = new char [strlen(initName) + 1];
    strcpy(name, initName);
} // DbDirDirectory::DbDirDirectory

// --------------------------------------------------------

DbDirDirectory::DbDirDirectory(const DbDirDirectory& rhs)
{
    if (this == &rhs)
    {
        return;
    } // if

} // DbDirDirectory::DbDirDirectory

// --------------------------------------------------------


DbDirDirectory::~DbDirDirectory()
{
    delete name;
    delete models;
    delete firstChild;
    delete sibling;
} // DbDirDirectory::~DbDirDirectory

// --------------------------------------------------------

DbDirDirectory::AddModelResponse DbDirDirectory::AddModel(char * path_name)
{
    char * pP = path_name;
    char * pP1;
    char subname[128];
    PathNameType nameType;
    DbDirModel * pM;
    DbDirModel * pNewModel;

    if (*pP != '/')
    {
        return AddModelBadPath;
    } // if
    pP++;
    DbDirDirectory * pD = this;
    for (;;)
    {
        nameType = nextName(pP, pP1, subname);
        switch (nameType)
        {
        case PathNameDirectoryEntry:
            pD = pD->firstChild->findNextDirectory(subname);
            if (pD == NULL)
            {
                return AddModelSubDirNotExist;
            } // if
            pP = pP1;
            pP++;
            break;
        case PathNameModelEntry:
            pM = pD->models->findNextModel(subname);
            if (pM != NULL) 
            {
                return AddModelModelExists; 
            } // if
            pNewModel = new DbDirModel(pD->models, subname, 0);
            models = pNewModel;
            return AddModelSuccess;
            break;
        NameBadName:
            return AddModelBadPath;
            break;
        default:
            return AddModelFailure;
            break;
        } // switch
    } // for
} // DbDirDirectory::AddModel

// --------------------------------------------------------

DbDirDirectory::PathNameType DbDirDirectory::nextName(char *pPin, char * pPout, char * subname)
{
    char * pP = pPin;
    char * pPreturn = subname;
    if (pPreturn == NULL) 
    {
        return PathNameError;
    } // if
    for (;;)
    {
        if (*pP == ' ')
        {
            return PathNameBadName;
        } // if
        else if (*pP == '/') 
        {
            pPout = pP;
            *pPreturn = '\0';
            return PathNameDirectoryEntry;
        } // else if
        else if (*pP == '\0') 
        {
            pPout = pP;
            *pPreturn = '\0';
            return PathNameModelEntry;
        } // else if
        else
        {
            *pPreturn = *pP;
            pP++;
        } // else
    } // for    
} // nextName

// --------------------------------------------------------

DbDirDirectory * DbDirDirectory::findNextDirectory(char * subname)
{
    DbDirDirectory * pD = this;
    while (pD != NULL) 
    {
        if (strcmp(pD->name, subname) == 0)
        {
            return pD;
        } // if
        pD = pD->sibling;
    } // while
    return NULL;
} // DbDirDirectory::findNextDirectory

// --------------------------------------------------------

DbDirModel * DbDirModel::findNextModel(char * subname)
{
    DbDirModel * pM = this;
    while (pM != NULL) 
    {
        if (strcmp(pM->name, subname) == 0)
        {
            return pM;
        } // if
        pM = pM->next;
    } // while
    return NULL;
} // DbDirModel::findNextModel

// --------------------------------------------------------

DbDirDirectory &DbDirDirectory::operator =(const DbDirDirectory& rhs)
{
    if (this == &rhs)
    {
        return *this;
    } // if

    return *this;
} // DbDirDirectory::operator =

// --------------------------------------------------------

// --------------------------------------------------------

DbDirModel::DbDirModel()
:   name(NULL), next(NULL), firstBlock(0)
{
} // DbDirModel::DbDirModel

// --------------------------------------------------------

DbDirModel::DbDirModel(DbDirModel * initNext, char * initName, Db::RecNumber initRecno)
:   next(initNext), firstBlock(initRecno)
{
    name = new char [strlen(initName) + 1];
    strcpy(name, initName);
} // DbDirModel::DbDirModel

// --------------------------------------------------------

DbDirModel::DbDirModel(const DbDirModel& rhs)
{
    if (this == &rhs)
    {
        return;
    } // if

} // DbDirModel::DbDirModel

// --------------------------------------------------------

DbDirModel::~DbDirModel()
{
    delete name;
} // DbDirModel::~DbDirModel

// --------------------------------------------------------

DbDirModel &DbDirModel::operator =(const DbDirModel& rhs)
{
    if (this == &rhs)
    {
        return *this;
    } // if

    return *this;
} // DbDirModel::operator =

// --------------------------------------------------------

