// krc.cpp    Knowledge representation compiler/decompiler.

/*
* Send comments to:
*   Dave Kuhlman
*   Reify
*   426 Madison Ave #9
*   Redwood City, CA 94061
*   415-368-8450
*   Internet: dkuhlman@netcom.com
*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#include <conio.h>
#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>

#include "mdl.hpp"
#include <db.hpp>
#include <bt.h>
#include <dbblk.hpp>
#include <mdlsyn.hpp>
#include "krstrg.hpp"
#include "krstrm.hpp"
#include "krg.h"
#include "kr.hpp"
// #include "charbuf.h"
// #include "ast.h"
#include "kr.h"

// ---------------------------------------------------------

static Boolean quietMode = False;

enum ACTIONS {
    ACT_ADD = 1,
    ACT_SHOWVERBOSE,
    ACT_DECOMPILE,
    ACT_CREATEDB,
    ACT_DELETEOBJECT
};

// ---------------------------------------------------------

// Global function prototypes:

#ifdef __cplusplus
extern "C" {
#endif

int getopt(
    int argc,
    char **argv,
    char *optstring
);

// void Parse(char * infilename);

#ifdef __cplusplus
}
#endif

// Local function prototypes:

static Boolean getKrArgs(int argc, char ** argv, int & retAction);
static void usage();
static void printMsg( char *msg );
static void createDb(char * dbName);
static void compileFile(char * dbName, char * inFileName, char * objectName);
static void showVerbose(char * dbname);
static void decompile(char * dbName, char * outfile, char * object);
static void deleteObject(char * dbname, char * objectname);

// ---------------------------------------------------------

// Variables for communication with getopt.

#ifdef __cplusplus
extern "C" {
#endif
extern  char *optarg;
extern  int optind;
#ifdef __cplusplus
}
#endif

// ---------------------------------------------------------

void main(int argc, char ** argv)
{
    int action;

    if (! getKrArgs(argc, argv, action))
    {
        exit(-1);
    } // if

    btInitialize();

    switch (action)
    {
    case ACT_CREATEDB:
        if ((argc - optind ) != 1)
        {
            usage();
            break;
        } // if
        createDb(argv[optind]);
        break;
    case ACT_ADD:
        if ((argc - optind ) != 3)
        {
            usage();
            break;
        } /* if */
        compileFile(argv[optind], argv[optind + 1], argv[optind + 2]);
        break;
    case ACT_SHOWVERBOSE:
        if ((argc - optind ) != 1)
        {
            usage();
            break;
        } /* if */
        showVerbose(argv[optind]);
        break;
    case ACT_DECOMPILE:
        if ((argc - optind ) != 3)
        {
            usage();
            break;
        } /* if */
        decompile(argv[optind], argv[optind + 1], argv[optind + 2]);
        break;
    case ACT_DELETEOBJECT:
        if ((argc - optind ) != 2)
        {
            usage();
            break;
        } /* if */
        deleteObject(argv[optind], argv[optind + 1]);
        break;
    default:
        break;
    } // switch

    btTerminate();
} // main

// ---------------------------------------------------------

// Get the options.

static Boolean getKrArgs(int argc, char ** argv, int & retAction)
{
    int     action = -1;
    int     factor;
    char    chr1;
    int     ichr1;

    if (argc <= 1)
    {
        usage();
        return(False);
    } // if

    while (True)
    {
        // Must cast to an int.
        ichr1 = getopt(argc, argv, "cvaxdq");
        if ( ichr1 == EOF )
        {
            break;
        } // if

        chr1 = (char)ichr1;
        switch ( chr1 )
        {
        case 'a':
            action = ACT_ADD;
            break;
        case 'q':
            quietMode = True;
            break;
        case 'v':
            action = ACT_SHOWVERBOSE;
            break;
        case 'x':
            action = ACT_DECOMPILE;
            break;
        case 'c':
            action = ACT_CREATEDB;
            break;
        case 'd':
            action = ACT_DELETEOBJECT;
            break;
        default:
            break;
        } // switch chr1
    } // while

    if ( action != -1 )
    {
        retAction = action;
        return(true);
    } // if
    else
    {
        usage();
        return(false);
    } // else
} // getKrArgs

// ---------------------------------------------------------

typedef char UsageLine[80];

UsageLine usageArray[] =
{
    "͸\n",
    "             KRC    Knowledge Representation Compiler               \n",
    "Usage:                                                              \n",
    "  KRC -c dbName                                                     \n",
    "  KRC -a dbName asciiInFile objectName                              \n",
    "  KRC -x dbName asciiOutFile objectName                             \n",
    "  KRC -v dbName                                                     \n",
    "Actions:                                                            \n",
    "  -c -- create database                                             \n",
    "  -a -- compile text file and add the object to the database        \n",
    "  -x -- decompile an object in a database to a text file            \n",
    "  -v -- show a verbose list of the objects in the database          \n",
    "Options:                                                            \n",
    "  -q -- quiet mode                                                  \n",
    ";",
    ""
};

static void usage()
{
    UsageLine   *pStr;
    int count;

    for (   pStr = usageArray, count = 0;
            (*pStr)[0] != '\0';
            pStr++, count++ )
    {
        printMsg( *pStr );
        if ( count > 22 )
        {
            printMsg(
    "*** Press any key to continue ***;"
            );
            getch();
            printf( "\r" );
            count = 0;
        } // if
    } // for

} // usage

// ---------------------------------------------------------

static void printMsg( char *msg )
{
    if ( ! quietMode )
    {
        printf( msg );
    } // if
} // printMsg

// ---------------------------------------------------------

static void createDb(char * dbName)
{
    btCreateDb(dbName);
} // createDb

// ---------------------------------------------------------

static void compileFile(char * dbName, char * inFileName, char * objectName)
{
    Bibliography * pBibliography;
    Db::RecNumber recno;

    pBibliography = Compile(inFileName, 0);

    if (pBibliography == NULL)
    {
        return;
    } // if

    DbBlock blk;
    if (! btOpenDb(dbName))
    {
        cout << "Can't open database: " << dbName << endl;
        return;
    } // if


    // If the object is already in the database, delete it.
    recno = btSearchKey(objectName);
    if (recno != Db::DB_NULL_RECID)
    {
        // Remove the records containing the data from the database.
        blk.DeleteDbRecs(recno);
        // Remove the key from the index/BTree.
        btDeleteKey(objectName);
    } // if

    // Add the object to the database.
    recno = pBibliography->Save(&blk, Db::DB_NULL_RECID);
    Db::RecNumber recno1 = blk.FlushBlocks(recno);
    btAddKey(objectName, recno1);
    btCloseDb();

    delete pBibliography;
} // compileFile

// ---------------------------------------------------------

static void decompile(char * dbName, char * outFileName, char * objectName)
{
    if (! btOpenDb(dbName))
    {
        cout << "Can't open database: " << dbName << endl;
        return;
    } // if

    DbBlock * pBlk = new DbBlock;
    Db::RecNumber nextRecno;

    // Search for the object by name.  Get the record number.
    Db::RecNumber recno = btSearchKey(objectName);
    if (recno == Db::DB_NULL_RECID)
    {
        cout << "*** Error.  Can't find key: " << objectName << endl;
        return;
    } // if

    // Load the records containing the object.
    recno = pBlk->LoadBlocks(recno);

    // Load the object from the DB.
    Bibliography * pBibliography = Bibliography::Load(pBlk, recno, nextRecno);

    // Decompile the bibliography.
    KrOstream ofs(outFileName);
    int count = 0;
    pBibliography->Decompile(ofs, 0, count);
    ofs.close();

    btCloseDb();
    delete pBibliography;
    delete pBlk;
} // decompile

// ---------------------------------------------------------

// Delete an object by name.
static void deleteObject(char * dbName, char * objectName)
{
    if (! btOpenDb(dbName))
    {
        cout << "Can't open database: " << dbName << endl;
        return;
    } // if

    DbBlock * pBlk = new DbBlock;

    // Search for the object by name.  Get the record number.
    Db::RecNumber recno = btSearchKey(objectName);
    if (recno == Db::DB_NULL_RECID)
    {
        cout << "*** Error.  Can't find key: " << objectName << endl;
        return;
    } // if

    // Remove the records containing the data from the database.
    pBlk->DeleteDbRecs(recno);

    // Remove the key from the index/BTree.
    btDeleteKey(objectName);

    btCloseDb();
    delete pBlk;
} // deleteObject

// ---------------------------------------------------------

void showVerbose(char * dbname)
{
    Db::RecNumber   recno;
    char            key[Db::MAX_KEY_SIZE];
    char            prevKey[Db::MAX_KEY_SIZE];

    if (! btOpenDb(dbname))
    {
        cout << "Can't open database: " << dbname << endl;
        return;
    } // if

    recno = btFirstKey( key );
    if ( recno == Db::DB_NULL_RECID )
    {
        printMsg( "Database is empty\n" );
        return;
    } // if
    cout << "Contents -- Keys:" << endl;
    cout << "    " << key << endl;
    strcpy( prevKey, key );
    recno = btNextKey( key );
    while   (
                ( recno != Db::DB_NULL_RECID ) &&
                ( strcmp( prevKey, key ) != 0 )
            )
    {
        cout << "    " << key << endl;
        strcpy( prevKey, key );
        recno = btNextKey( key );
    } // while
    btCloseDb();
} // showVerbose

// ---------------------------------------------------------

void KrReportError(char *msg, int level)
{
    switch (level)
    {
    case 1:
        cerr << msg << endl;
        break;
    case 2:
        cerr << msg << endl;
        break;
    case 3:
        cerr << msg << endl;
        break;
    case 4:
        cerr << msg << endl;
        break;
    default:
        cerr << "(KrReportError) Bad error level: " << level << endl;
        break;
    } // switch
} // KrReportError

// ---------------------------------------------------------
