// krstrg.hpp    Class definitions for KrString

/*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/


#ifndef krstrg_HPP
#define krstrg_HPP

#if defined(__WATCOMC__)
#include <string.hpp>
#elif defined(__IBMCPP__)
#include <istring.hpp>
#else
#include <cstring.h>
#endif // __WATCOM__
#include "krstrm.hpp"
#include "db.hpp"
#include "dbblk.hpp"
#include "mdlsyn.hpp"

class KrString;
struct DbKrString;
typedef Db::RecNumber DbKrStringPtr;


class KrStringLink {
public:
    KrStringLink() : data(NULL), next(NULL) { }
    KrStringLink(KrString * initData)
        : data(initData), next(NULL)
        { }
    KrStringLink(KrString * initData, KrStringLink * initNext)
        : data(initData), next(initNext)
        { }
    KrStringLink(const KrStringLink & rhs);
    ~KrStringLink() { delete next; };
    KrString * GetData(void) { return data; };
    KrStringLink * GetNext(void) { return next; };
    void SetNext(KrStringLink * newNext) { next = newNext; }
    KrStringLink * Add(KrString * newItem)
    {
        return new KrStringLink(newItem, this);
    } // Add
    KrStringLink & operator =(const KrStringLink & rhs)
    {
        if (this == &rhs)
        {
            return *this;
        } // if
        data = rhs.data;
        next = rhs.next;
        return *this;
    } // KrStringLink::operator =
protected:
private:
    KrString * data;
    KrStringLink *next;
}; // KrStringLink


class KrStringList {
public:
    KrStringList() : first(NULL), last(NULL) { }
    KrStringList(KrString * initData) {
        first = new KrStringLink(initData);
        last = first;
    }
    void Append(KrString * initData);
    void Insert(KrString * initData, int pos);
    boolean Delete(int pos);
    KrStringLink * GetFirst() { return first; }
    KrStringLink * GetLast() { return last; }
    KrString * operator [](int pos);
    KrStringList * Copy();
    static KrStringList * Load(DbBlock * block, Db::RecNumber firstRecno);
    Db::RecNumber Save(DbBlock * block);
protected:
private:
    KrStringLink *first;
    KrStringLink *last;
}; // KrStringList


#if defined(__WATCOMC__)
class KrString : public String {
#elif defined(__IBMCPP__)
class KrString : public IString {
#else // __WATCOMC__
class KrString : public string {
#endif // __WATCOMC__
public:
    KrString();
    KrString(char * initStr);
    KrString(const KrString& rhs);
    KrString(DbBlock * block, Db::RecNumber recno);
    ~KrString();
    KrString& operator =(const KrString& rhs);

    // None of these worked.
    // operator char *();
    // operator char const *() { return rep->c_str(); }
    // operator char *() { return (const char *)rep->c_str(); }

    // Problem: This one works, but it casts away const-ness.
#if defined(__WATCOMC__)
    operator char *() { return (char *)(const char *)*rep; }
#elif defined(__IBMCPP__)
    operator char *() { return (char *)(const char *)*rep; }
#else // __WATCOMC__
    operator char *() { return (char *)rep->c_str(); }
#endif // __WATCOMC__

    void Decompile(KrOstream& outf, int level, int & count);
    void EscapeChars(char * buf, int maxBufLen);
    // KrString* Load(Db::RecNumber recno);
    KrString& Copy(void);
    Db::RecNumber Save(DbBlock * block);
    // Function with prevRecno to make code generation more consistent.
//    static KrStringList * CopyList(KrStringList * pOldList);
//    static KrStringList * LoadList(DbBlock * block, Db::RecNumber firstRecno);
//    static Db::RecNumber SaveList(DbBlock * block, KrStringList * plistKrString);
protected:
private:
#if defined(__WATCOMC__)
    String * rep;
#elif defined(__IBMCPP__)
    IString * rep;
#else // __WATCOMC__
    string * rep;
#endif // __WATCOMC__
}; // KrString

struct DbString {
    Db::RecNumber next;
    Db::RecNumber data;
}; // DbString


#endif // krstrg_HPP

